@@ -3,6 +3,9 @@
 # Set encoding default for python 2.7
 # vim: syntax=python noexpandtab
 
+# ToDo
+# 1. export config_id and config_name in osdesc
+
 # gadgetconfig configures and enables a Gadget USB configuration
 #
 # N.B. annotations are supported by adding comments attributes:
@@ -31,7 +34,6 @@ import sys
 import collections
 import fnmatch
 import re
-from scandir import scandir, walk
 
 """gadget.py: ..."""
 
@@ -46,7 +48,7 @@ class ExportGadget(object):
 		self.verbose = True
 		# device['# E.g.: usb_f_acm, usb_f_ecm, usb_f_eem, usb_f_hid, usb_f_mass_storage'] = ''
 		# device['#       usb_f_midi, usb_f_ncm, usb_f_obex, usb_f_rndis, usb_f_serial'] = ''
-		self.interfaces = {'acm':2, 'ecm':2, 'eem':1, 'ncm':2, 'hid': 1, 'mass_storage': 1, 'rndis':1, 'serial': 2 }
+		self.interfaces = {'acm': 2, 'ecm': 2, 'eem': 1, 'ncm': 2, 'hid': 1, 'mass_storage': 1, 'rndis': 1, 'serial': 2}
 
 	def pathread(self, path):
 
@@ -71,7 +73,8 @@ class ExportGadget(object):
 			except (PermissionError, OSError):
 				return ''
 			except UnicodeDecodeError:
-				print('pathread: [UnicodeDecodeError]', file=sys.stderr)
+				# print('pathread: [UnicodeDecodeError]', file=sys.stderr)
+				pass
 
 			try:
 				f = open(path, "rb")
@@ -88,7 +91,7 @@ class ExportGadget(object):
 	def export_attribute_entry(self, path, entry, symlinks=False, exclude=[]):
 		if any(fnmatch.fnmatch(entry, pattern) for pattern in exclude):
 			return None
-		print("export_attributes: entry %s" % (entry), file=sys.stderr)
+		# print("export_attributes: entry %s" % (entry), file=sys.stderr)
 		epath = "%s/%s" % (path, entry)
 		if symlinks and os.path.islink(epath):
 			realpath = os.path.realpath(epath)
@@ -101,9 +104,9 @@ class ExportGadget(object):
 			a = []
 			for b in data:
 				a.append("0x%02x" % (b))
-			print("export_attribute_entry: b: %s" % (a), file=sys.stderr)
+			# print("export_attribute_entry: b: %s" % (a), file=sys.stderr)
 			return a
-		print("data: %s" % (data), file=sys.stderr)
+		# print("data: %s" % (data), file=sys.stderr)
 		if data is None or len(data) == 0:
 			return
 		return data[0].rstrip('\t\r\n\0')
@@ -161,9 +164,9 @@ class ExportGadget(object):
 
 			# we need to iterate across the config directory twice, first to get
 			# the sorted attributes and symlink targets. Then a second time
-			# to get the symlink creation times so we can generate the 
+			# to get the symlink creation times so we can generate the
 			# functions list in the order they were created in. This appears
-			# to work reliably. 
+			# to work reliably.
 			#
 			function_map = {}
 			functions = []
@@ -202,31 +205,30 @@ class ExportGadget(object):
 
 			# config_entries = sorted(os.listdir(config_path), key=str.casefold)
 			config_dirents = sorted(os.scandir(config_path), key=lambda dirent: dirent.inode())
-			print("************\nexport_device_configs: config_entries %s" % (config_entries), file=sys.stderr)
+			# print("************\nexport_device_configs: config_entries %s" % (config_entries), file=sys.stderr)
 			num = 0
 			interface = 0
 			valid = True
-			idVendor = re.sub(r'0x(.*)',r'\1',idVendor)
-			idProduct = re.sub(r'0x(.*)',r'\1',idProduct)
-			print("****\nexport_device_configs: len(config_dirents) %s" % (len(config_dirents)), file=sys.stderr)
+			idVendor = re.sub(r'0x(.*)', r'\1', idVendor)
+			idProduct = re.sub(r'0x(.*)', r'\1', idProduct)
+			# print("****\nexport_device_configs: len(config_dirents) %s" % (len(config_dirents)), file=sys.stderr)
 			symlinks = 0
 			for dirent in config_dirents:
 				if dirent.is_symlink(): symlinks += 1
 			for dirent in config_dirents:
 				if dirent.is_symlink():
-					print("export_device_configs: name: %s" % (dirent.name), file=sys.stderr)
+					# print("export_device_configs: name: %s" % (dirent.name), file=sys.stderr)
 					if symlinks > 1:
 						a = "# Host Match USB\\VID_%s&PID_%s&MI_%02d" % (idVendor.upper(), idProduct.upper(), interface)
 					else:
 						a = "# Host Match USB\\VID_%s&PID_%s" % (idVendor.upper(), idProduct.upper())
 					functions.append({a: '', 'name': dirent.name, 'function': function_map[dirent.name]})
 					# (f,n) = dirent.name.split('.')
-					(f,n) = function_map[dirent.name].split('.')
+					(f, n) = function_map[dirent.name].split('.')
 					if valid and f in self.interfaces:
 						interface += self.interfaces[f]
 					else:
 						valid = False
-						interfaces = ''
 					num += 1
 
 			config['# This determines the order in the Configuration descriptor'] = ''
@@ -241,23 +243,13 @@ class ExportGadget(object):
 		# print("export_function_os_desc: os_desc_path: %s" % (os_desc_path), file=sys.stderr)
 		os_desc = {}
 		os_desc_entries = sorted(os.listdir(os_desc_path), key=str.casefold)
+
 		for entry in os_desc_entries:
 			epath = "%s/%s" % (os_desc_path, entry)
 			if os.path.isdir(epath) and not os.path.islink(epath) and fnmatch.fnmatch(entry, "interface.*"):
 				os_desc[entry] = self.export_attributes(epath)
 		return os_desc
 
-	def export_function_lun(self, lun_path):
-		# print("", file=sys.stderr)
-		print("export_function_lun: lun_path: %s" % (lun_path), file=sys.stderr)
-		lun = {}
-		lun_entries = sorted(os.listdir(lun_path), key=str.casefold)
-		for entry in lun_entries:
-			epath = "%s/%s" % (lun_path, entry)
-			if os.path.isdir(epath) and not os.path.islink(epath) and fnmatch.fnmatch(entry, "interface.*"):
-				lun[entry] = self.export_attributes(epath)
-		return os_desc
-
 	def export_device_functions(self, functions_path, annotation=None):
 		# print("", file=sys.stderr)
 		# print("export_device_functions: functions_path: %s" % (functions_path), file=sys.stderr)
@@ -273,7 +265,7 @@ class ExportGadget(object):
 					if entry == 'os_desc':
 						function[entry] = self.export_function_os_desc(epath)
 					if fnmatch.fnmatch(entry, 'lun.*'):
-						print("export_device_functions: %s" % (entry), file=sys.stderr)
+						# print("export_device_functions: %s" % (entry), file=sys.stderr)
 						function[entry] = self.export_attributes(epath)
 					continue
 				elif os.path.islink(epath):
@@ -295,7 +287,28 @@ class ExportGadget(object):
 
 	def export_device_os_desc(self, os_desc_path):
 		# print("export_device_os_desc: os_desc_path: %s" % (os_desc_path), file=sys.stderr)
-		os_desc = self.export_attributes(os_desc_path, symlinks=True)
+		os_desc = self.export_attributes(os_desc_path, symlinks=False)
+		# print("export_device_os_desc: os_desc: %s" % (os_desc), file=sys.stderr)
+
+		if 'use' not in os_desc:
+			# print("NO USE", file=sys.stderr)
+			return None
+		if os_desc['use'] != '1':
+			# print("USE != 1", file=sys.stderr)
+			return None
+
+		# print("USE == 1", file=sys.stderr)
+		for e in os.listdir(os_desc_path):
+			if not os.path.islink("%s/%s" % (os_desc_path, e)):
+				# print("export_device_os_desc: e: %s NOT" % (e), file=sys.stderr)
+				continue
+			# print("export_device_os_desc: e: %s" % (e), file=sys.stderr)
+
+			config_name, config_id = e.split(".")
+			os_desc['config_name'] = config_name
+			os_desc['config_id'] = config_id
+			# print("export_device_os_desc: config_name: %s config_id: %s" % (config_name, config_id), file=sys.stderr)
+
 		return os_desc
 
 	def export_devices(self, annotations=True):
@@ -305,7 +318,7 @@ class ExportGadget(object):
 			device_path = "%s/%s" % (self.configpath, device_name)
 			device = self.export_attributes(device_path, exclude=['UDC'], idFlag=True,
 					annotation={'# USB Device Descriptor Fields': ''})
-			print("export_device: device: %s" % (device), file=sys.stderr)
+			# print("export_device: device: %s" % (device), file=sys.stderr)
 			# print("export_device: device_path: %s" % (device_path), file=sys.stderr)
 			device_entries = sorted(os.listdir(device_path), key=str.casefold)
 			if 'strings' in device_entries:
@@ -333,9 +346,11 @@ class ExportGadget(object):
 					elif entry == 'functions':
 						pass
 					elif entry == 'os_desc':
-						device['# Microsoft OS Descriptors Support'] = ''
-						device['# C.f. https://docs.microsoft.com/en-us/previous-versions/gg463179(v=msdn.10)'] = ''
-						device[entry] = self.export_device_os_desc(epath)
+						os_desc = self.export_device_os_desc(epath)
+						if os_desc is not None:
+							device['# Microsoft OS Descriptors Support'] = ''
+							device['# C.f. https://docs.microsoft.com/en-us/previous-versions/gg463179(v=msdn.10)'] = ''
+							device[entry] = os_desc
 					elif entry == 'strings':
 						pass
 				elif os.path.islink(epath):
