@@ -23,17 +23,26 @@ except ModuleNotFoundError:
 
 class ManageGadget(object):
 
-	def __init__(self, configpath, test=False):
+	def __init__(self, configpath, test=False, verbose=False, auto_serialnumber=False):
 		self.configpath = configpath
 		self.udcpath = "/sys/class/udc"
 		self.realudcpath = ''
 		self.udclist = []
 		self.test = test
 		self.configured_device = ''
+		self.verbose = verbose
+		self.auto_serialnumber = auto_serialnumber
+		# print("ManageGadget: verbose: %s" % (verbose))
 
-		self.checkfs(verbose=True)
-		self.find_udcs(verbose=True)
-		self.check_current(verbose=True)
+		self.checkfs(verbose=verbose)
+		self.find_udcs(verbose=verbose)
+		self.check_current(verbose=verbose)
+
+
+
+	def vprint(self, s):
+		if self.verbose:
+			print(s, file=sys.stderr)
 
 	def pathread(self, path):
 
@@ -61,25 +70,21 @@ class ManageGadget(object):
 
 		return '<UNKNOWN>'
 
-	def vprint(self, verbose, s):
-		if verbose:
-			print(s, file=sys.stderr)
-
 	# checkfs will verify that the usb_gadget configfs is available
 	#
 	def checkfs(self, verbose=False):
 		try:
 			if not os.path.isdir(self.configpath):
-				self.vprint(verbose, "%s: not a directory" % (self.configpath))
+				self.vprint("%s: not a directory" % (self.configpath))
 				return False
 			if os.path.islink(self.configpath):
-				self.vprint(verbose, "%s: is a symlink to %s" % (self.configpath, os.path.realpath(self.configpath)))
+				self.vprint("%s: is a symlink to %s" % (self.configpath, os.path.realpath(self.configpath)))
 				return False
 			return True
 		except PermissionError:
-			self.vprint(verbose, "Cannot read: %s PermissionError" % (self.configpath))
+			self.vprint("Cannot read: %s PermissionError" % (self.configpath))
 		except NameError:
-			self.vprint(verbose, "Cannot read: %s NameError" % (self.configpath))
+			self.vprint("Cannot read: %s NameError" % (self.configpath))
 		return False
 
 	# def detach(self):
@@ -92,16 +97,16 @@ class ManageGadget(object):
 			if os.path.islink(fpath):
 				self.udclist.append(l)
 				self.realudcpath = os.path.realpath(fpath)
-				self.vprint(verbose, "  %s -> %s" % (l, self.realudcpath))
+				self.vprint("  %s -> %s" % (l, self.realudcpath))
 			else:
-				self.vprint(verbose, "  %s <UNKNOWN>" % (l))
-		self.vprint(verbose, "")
+				self.vprint("  %s <UNKNOWN>" % (l))
+		self.vprint("")
 		return self.udclist
 
 	def _query_gadget(self, all=False):
 		gadgets = []
 		dirs = os.listdir(self.configpath)
-		self.vprint(True, "Current Gadget configurations")
+		self.vprint("Current Gadget configurations")
 		if len(dirs) == 0:
 			return gadgets
 		for d in dirs:
@@ -133,7 +138,7 @@ class ManageGadget(object):
 	def query_gadgets(self):
 		gadgets = []
 		dirs = os.listdir(self.configpath)
-		self.vprint(True, "Current Gadget configurations")
+		self.vprint("Current Gadget configurations")
 		if len(dirs) == 0:
 			return gadgets
 		for d in dirs:
@@ -170,9 +175,9 @@ class ManageGadget(object):
 
 	def check_current(self, verbose=False):
 		dirs = os.listdir(self.configpath)
-		self.vprint(verbose, "Current Gadget configurations")
+		self.vprint("Current Gadget configurations")
 		if len(dirs) == 0:
-			self.vprint(verbose, "<NONE>")
+			self.vprint("<NONE>")
 
 		# print("check_current: %s" % (self.configpath))
 		# print("check_current: dirs: %s" % (dirs))
@@ -181,11 +186,11 @@ class ManageGadget(object):
 			# print("udc: type: %s len: %s" % (type(udc),len(udc)))
 			if len(udc) == 1 and len(udc[0]) > 1:
 				# print("udc: type: %s len: %s" % (type(udc[0]),len(udc[0])))
-				self.vprint(verbose, "  %s UDC -> %s" % (d, udc[0].rstrip()))
+				self.vprint("  %s UDC -> %s" % (d, udc[0].rstrip()))
 				self.configured_device = d
 			else:
-				self.vprint(verbose, "  %s Not Configured" % (d))
-		self.vprint(verbose, "")
+				self.vprint("  %s Not Configured" % (d))
+		self.vprint("")
 
 	# update UDC file to enable or disable a Gadget
 	def update_udc(self, name, s):
@@ -199,7 +204,6 @@ class ManageGadget(object):
 			print("update_udc: %s File Not Found Error" % (udcpath), file=sys.stderr)
 			exit(1)
 
-
 	# write \n to UDC to disable a Gadget
 	def disable_current(self):
 		if self.query_gadget() is None:
@@ -231,7 +235,7 @@ class ManageGadget(object):
 				# print("UDC already not attached!")
 				return
 		try:
-			op = ['disconnect', 'connect'][flag]
+			# op = ['disconnect', 'connect'][flag]
 			# print("udclist: %s op: %s" % (self.udclist, op), file=sys.stderr)
 			f = open("/sys/class/udc/%s/soft_connect" % (self.udclist[0]), 'w')
 			f.write("%s\n" % (['disconnect', 'connect'][flag]))
@@ -253,7 +257,8 @@ class ManageGadget(object):
 		with io.open(pathname) as f:
 			try:
 				device_definitions = commentjson.load(f)
-			except (UnexpectedCharacters):
+			# except (UnexpectedCharacters):
+			except:
 				print("get_device_name: Unexpected Characters")
 				exit(1)
 			names = []
@@ -263,7 +268,7 @@ class ManageGadget(object):
 
 	# check_device_file
 	def check_device_file(self, pathname, device_name=None, args=None):
-		print("check_device_file: pathname: %s device_name %s" % (pathname, device_name))
+		# print("check_device_file: pathname: %s device_name %s" % (pathname, device_name))
 		try:
 			f = io.open(pathname)
 		except (FileNotFoundError):
@@ -271,7 +276,8 @@ class ManageGadget(object):
 			exit(1)
 		try:
 			device_definitions = commentjson.load(f)
-		except (UnexpectedCharacters):
+		# except (UnexpectedCharacters):
+		except:
 			print("check_device_file: Unexpected Characters")
 			exit(1)
 
@@ -292,18 +298,19 @@ class ManageGadget(object):
 					data[k] = repl
 				self.replace(v, match, repl)
 
-	def add_device_file(self, pathname, new_device_name=None, args=None):
-		print("*****\nadd_device_file: path: %s new_device_name: %s" % (pathname, new_device_name))
-		a = AddGadget(self.configpath)
+	def add_device_file(self, pathname, new_device_name=None, args=None, sh=False, enable=False, auto_serialnumber=None):
+		# print("*****\nadd_device_file: path: %s new_device_name: %s" % (pathname, new_device_name), file=sys.stdeff)
+		a = AddGadget(self.configpath, pathname=pathname, verbose=self.verbose, sh=sh, enable=enable)
 		try:
 			f = io.open(pathname)
 		except (FileNotFoundError):
-			print("add_device_file: File Not Found Error", file=sys.stderr)
+			self.vprint("add_device_file: File Not Found Error")
 			exit(1)
 		try:
 			device_definitions = commentjson.load(f)
-		except (UnexpectedCharacters):
-			print("add_device_file: Unexpected Characters")
+		# except (UnexpectedCharacters):
+		except:
+			self.vprint("add_device_file: Unexpected Characters")
 			exit(1)
 
 		for definition_name in device_definitions:
@@ -312,8 +319,7 @@ class ManageGadget(object):
 				device_name = new_device_name
 			else:
 				device_name = definition_name
-			# print('device_name: %s' % (device_name))
-			if device_name in self.query_gadgets():
+			if not sh and device_name in self.query_gadgets():
 				# print("add_device_file: %s already defined" % (device_name))
 				continue
 
@@ -326,5 +332,16 @@ class ManageGadget(object):
 				if args.dev_addr: self.replace(device_definition, 'dev_addr', args.dev_addr)
 				if args.host_addr: self.replace(device_definition, 'host_addr', args.host_addr)
 
+			if self.auto_serialnumber or auto_serialnumber:
+				serialnumber_path = "/proc/device-tree/serial-number"
+				try:
+					f = open(serialnumber_path)
+					lines = f.readlines(1)
+					serialnumber = lines[0]
+					serialnumber = serialnumber[:-1]
+					self.replace(device_definition, 'serialnumber', serialnumber)
+				except:
+					pass
+
 			a.add_device_json(device_definition, device_name=device_name)
 
